@@ -9,6 +9,8 @@ use directories::{BaseDirs, ProjectDirs};
 use once_cell::sync::Lazy;
 use serde::Deserialize;
 
+use tor_error::{ErrorKind, HasKind};
+
 /// A path in a configuration file: tilde expansion is performed, along
 /// with expansion of certain variables.
 ///
@@ -63,6 +65,26 @@ pub enum CfgPathError {
     InvalidString(String),
 }
 
+impl HasKind for CfgPathError {
+    fn kind(&self) -> ErrorKind {
+        use CfgPathError as E;
+        use ErrorKind as EK;
+        match self {
+            E::UnknownVar(_) | E::InvalidString(_) => EK::InvalidConfig,
+            E::NoProjectDirs | E::NoBaseDirs => EK::NoHomeDirectory,
+            E::BadUtf8(_) => {
+                // Arguably, this should be a new "unsupported config"  type,
+                // since it isn't truly "invalid" to have a string with bad UTF8
+                // when it's going to be used as a filename.
+                //
+                // With luck, however, this error will cease to exist when shellexpand
+                // improves its character-set handling.
+                EK::InvalidConfig
+            }
+        }
+    }
+}
+
 impl CfgPath {
     /// Create a new configuration path
     pub fn new(s: String) -> Self {
@@ -172,6 +194,7 @@ mod test {
         assert_eq!(p.path().unwrap().to_str(), Some("/usr/local/foo"));
     }
 
+    #[cfg(not(target_family = "windows"))]
     #[test]
     fn expand_home() {
         let p = CfgPath::new("~/.arti/config".to_string());
@@ -185,6 +208,21 @@ mod test {
         assert_eq!(p.path().unwrap().to_str(), expected.to_str());
     }
 
+    #[cfg(target_family = "windows")]
+    #[test]
+    fn expand_home() {
+        let p = CfgPath::new("~\\.arti\\config".to_string());
+        assert_eq!(p.to_string(), "~\\.arti\\config".to_string());
+
+        let expected = dirs::home_dir().unwrap().join(".arti\\config");
+        assert_eq!(p.path().unwrap().to_str(), expected.to_str());
+
+        let p = CfgPath::new("${USER_HOME}\\.arti\\config".to_string());
+        assert_eq!(p.to_string(), "${USER_HOME}\\.arti\\config".to_string());
+        assert_eq!(p.path().unwrap().to_str(), expected.to_str());
+    }
+
+    #[cfg(not(target_family = "windows"))]
     #[test]
     fn expand_cache() {
         let p = CfgPath::new("${ARTI_CACHE}/example".to_string());
@@ -194,6 +232,16 @@ mod test {
         assert_eq!(p.path().unwrap().to_str(), expected.to_str());
     }
 
+    #[cfg(target_family = "windows")]
+    #[test]
+    fn expand_cache() {
+        let p = CfgPath::new("${ARTI_CACHE}\\example".to_string());
+        assert_eq!(p.to_string(), "${ARTI_CACHE}\\example".to_string());
+
+        let expected = project_dirs().unwrap().cache_dir().join("example");
+        assert_eq!(p.path().unwrap().to_str(), expected.to_str());
+    }
+
     #[test]
     fn expand_bogus() {
         let p = CfgPath::new("${ARTI_WOMBAT}/example".to_string());
